// Copyright 2004 Adam Megacz, see the COPYING file for licensing [GPL]
package org.ibex.js;

/**
 *  This class encapsulates a single trap placed on a given node. The
 *  traps for a given property name on a given box are maintained as a
 *  linked list stack, with the most recently placed trap at the head
 *  of the list.
 */
class Trap {

    JS trapee = null;          ///< the box on which this trap was placed
    JS name = null;        ///< the property that the trap was placed on

    JSFunction f = null;       ///< the function for this trap
    Trap next = null;          ///< the next trap down the trap stack

    Trap(JS b, JS n, JSFunction f, Trap nx) {
        trapee = b; name = n; this.f = f; this.next = nx;
    }

    static final JSFunction putInvoker = new JSFunction("putInvoker", 0, null);
    static final JSFunction getInvoker = new JSFunction("getInvoker", 0, null);

    static {
        putInvoker.add(1, ByteCodes.PUT, null);
        putInvoker.add(2, Tokens.RETURN, null);
        getInvoker.add(1, ByteCodes.GET, null);
        getInvoker.add(2, Tokens.RETURN, null);
    }
    
    boolean readTrap() { return f.numFormalArgs == 0; }
    boolean writeTrap() { return f.numFormalArgs != 0; }
    
    void invoke(JS value) throws JSExn {
        Interpreter i = new Interpreter(putInvoker, false, null);
        i.stack.push(trapee);
        i.stack.push(name);
        i.stack.push(value);
        i.resume();
    }

    JS invoke() throws JSExn {
        Interpreter i = new Interpreter(getInvoker, false, null);
        i.stack.push(trapee);
        i.stack.push(name);
        return i.resume();
    }
}
